"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMethodCfg = exports.invokeDataSource = void 0;
const fn_context_1 = require("./fn-context");
const builtin_method_1 = require("./builtin-method");
const http_method_1 = require("./http-method");
const cloudfn_method_1 = require("./cloudfn-method");
const utils_1 = require("./utils");
const config_1 = require("./config");
async function invokeDataSource(cfnParams) {
    // const fnParams = params.params;
    const { dataSource } = cfnParams;
    const defaultParams = config_1.getDefaultParams(dataSource.name);
    let fnParams = cfnParams.params || {};
    if (defaultParams) {
        fnParams = utils_1.mergeDeep(defaultParams, fnParams);
    }
    const context = fn_context_1.getSpecificContext({ dataSource });
    if (dataSource.type === 'database') {
        if (builtin_method_1.isBuiltinMethod(cfnParams.methodName)) {
            return builtin_method_1.callBuiltinMethod(Object.assign(Object.assign({}, cfnParams), { dataSourceName: dataSource.name }), context);
        }
    }
    const methodCfg = getMethodCfg(dataSource, cfnParams.methodName);
    if (!methodCfg) {
        throw new utils_1.TCBError(404, `method ${cfnParams.methodName} not found in datasource ${dataSource.name}`);
    }
    // 是否正在测试当前方法
    const isTestMode = config_1.isMethodInTestMode(dataSource.name, methodCfg.name);
    // 方法被禁用且未在数据源控制台中进行方法测试
    if (methodCfg.disabled && !isTestMode) {
        throw new utils_1.TCBError(403, `method ${cfnParams.methodName} not enabled in datasource ${dataSource.name}`);
    }
    if (methodCfg.type === 'http') {
        // TODO: 测试模式下, 数据源控制台 方法测试 功能验证
        return http_method_1.callHttpMethod(Object.assign(Object.assign({}, cfnParams), { params: fnParams, methodCfg,
            isTestMode }), context);
    }
    if (methodCfg.type === 'cloud-function') {
        return cloudfn_method_1.callCloudFnMethod(Object.assign(Object.assign({}, cfnParams), { params: fnParams, methodCfg }), context);
    }
}
exports.invokeDataSource = invokeDataSource;
/**
 * 获取方法配置描述
 */
function getMethodCfg(dsConfig, methodName) {
    if (dsConfig && Array.isArray(dsConfig.methods)) {
        return dsConfig.methods.find(item => (item === null || item === void 0 ? void 0 : item.name) === methodName);
    }
}
exports.getMethodCfg = getMethodCfg;
